@@ -7,7 +7,7 @@
 /* For the what command                 */
 /* ------------------------------------ */
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: Cpool.c,v $ $Revision: 3476 $ $Date: 2010-03-10 13:40:05 +0100 (Wed, 10 Mar 2010) $ CERN IT-ADC-CA/HSM Jean-Damien Durand";
+static char sccsid[] = "@(#)$RCSfile: Cpool.c,v $ $Revision: 10783 $ $Date: 2014-07-18 11:28:23 +0200 (Fri, 18 Jul 2014) $ CERN IT-ADC-CA/HSM Jean-Damien Durand";
 #endif /* not lint */
 
 #include <Cpool_api.h>
@@ -827,6 +827,7 @@ void *_Cpool_starter(arg)
 			/* Thread only			  */
 			/*----------------------- */
 			struct Cpool_t *current;
+			int		   first;
 			int 			index;
 			char		   *dummy;
 			void		  *(*start) _PROTO((void *));
@@ -862,6 +863,7 @@ void *_Cpool_starter(arg)
 			}
 #endif
 			current->state[index] = 0;
+			first = 1;
 #ifdef CPOOL_DEBUG
 			if (Cpool_debug != 0) {
 				log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : cond-signal on current->state_cthread_structure[%d]\n",
@@ -869,10 +871,25 @@ void *_Cpool_starter(arg)
 			}
 #endif
 			Cthread_cond_signal_ext(current->state_cthread_structure[index]);
+
+#ifdef CPOOL_DEBUG
+			if (Cpool_debug != 0) {
+				log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : un-lock on current->state_cthread_structure[%d]\n",
+					_Cpool_self(),_Cthread_self(),index);
+			}
+#endif
+			if (Cthread_mutex_unlock_ext(current->state_cthread_structure[index]) != 0) {
+#ifdef CPOOL_DEBUG
+				if (Cpool_debug != 0) {
+					log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : un-lock on current->state_cthread_structure[%d] error: %s\n",
+						_Cpool_self(),_Cthread_self(), index, strerror(errno));
+				}
+#endif
+				return(NULL);
+			}
 					
 			/* We loop indefinitely */
 			while (1) {
-			  
 #ifdef CPOOL_DEBUG
 				if (Cpool_debug != 0) {
 					log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : lock on lock_parent_cthread_structure\n",
@@ -885,7 +902,37 @@ void *_Cpool_starter(arg)
 					/* Error */
 					return(NULL);
 				}
-			  
+
+#ifdef CPOOL_DEBUG
+				if (Cpool_debug != 0) {
+					log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : lock on current->state_cthread_structure[%d]\n",
+						_Cpool_self(),_Cthread_self(),index);
+				}
+#endif
+				if (Cthread_mutex_lock_ext(current->state_cthread_structure[index]) != 0) {
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : lock on current->state_cthread_structure[%d] error: %s\n",
+							_Cpool_self(),_Cthread_self(), index, strerror(errno));
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : un-lock on lock_parent_cthread_structure\n",
+							_Cpool_self(),_Cthread_self());
+					}
+#endif
+					Cthread_mutex_unlock_ext(lock_parent_cthread_structure);
+					return(NULL);
+				}
+
+				if (!first) {
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : Setting current->state[%d] to 0\n",
+							_Cpool_self(),_Cthread_self(),index);
+					}
+#endif
+					/* We flag us in the non-running state */
+					current->state[index] = 0;
+				}
+
 #ifdef CPOOL_DEBUG
 				if (Cpool_debug != 0) {
 					log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : current->flag is %d\n",
@@ -893,8 +940,8 @@ void *_Cpool_starter(arg)
 				}
 #endif
 			  
-				if (current->flag == -1) {
-					/* Either the parent is waiting for any of us, or for us only */
+				if (current->flag == -1 && current->state[index] == 0) {
+					/* The parent is waiting and we are free */
 #ifdef CPOOL_DEBUG
 					if (Cpool_debug != 0) {
 						log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : current->forceid is %d\n",
@@ -949,7 +996,7 @@ void *_Cpool_starter(arg)
 #endif
 			  
 				Cthread_mutex_unlock_ext(lock_parent_cthread_structure);
-			  
+
 				/* We wait until there is something new      */
 				/* The lock on "current->state[index]" is    */
 				/* then released                             */
@@ -1004,6 +1051,22 @@ void *_Cpool_starter(arg)
 				/* current->arg[index]                            */
 				start    = (void *(*) _PROTO((void *))) current->start[index];
 				startarg = (void *)            current->arg[index];
+
+#ifdef CPOOL_DEBUG
+				if (Cpool_debug != 0) {
+					log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : un-lock on current->state_cthread_structure[%d]\n",
+						_Cpool_self(),_Cthread_self(),index);
+				}
+#endif
+				if (Cthread_mutex_unlock_ext(current->state_cthread_structure[index]) != 0) {
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : un-lock on current->state_cthread_structure[%d] error: %s\n",
+							_Cpool_self(),_Cthread_self(), index, strerror(errno));
+					}
+#endif
+					return(NULL);
+				}
 			  
 #ifdef CPOOL_DEBUG
 				if (Cpool_debug != 0) {
@@ -1011,7 +1074,7 @@ void *_Cpool_starter(arg)
 						_Cpool_self(),_Cthread_self(),(unsigned long) start,(unsigned long) (startarg != NULL ? startarg : 0));
 				}
 #endif
-			  
+
 				/* We execute the routine                         */
 				(*start)(startarg);
 			  
@@ -1021,26 +1084,8 @@ void *_Cpool_starter(arg)
 						_Cpool_self(),_Cthread_self(),(unsigned long) start,(unsigned long) (startarg != NULL ? startarg : 0));
 				}
 #endif
-			  
-#ifdef CPOOL_DEBUG
-				if (Cpool_debug != 0) {
-					log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : Setting current->state[%d] to 0\n",
-						_Cpool_self(),_Cthread_self(),index);
-				}
-#endif
-				
-				/* We flag us in the non-running state */
-				current->state[index] = 0;
-
+				first = 0;
 			}
-
-#ifdef CPOOL_DEBUG
-			if (Cpool_debug != 0) {
-				log(LOG_INFO,"[Cpool  [%2d][%2d]] In _Cpool_starter : un-lock on current->state_cthread_structure[%d]\n",
-					_Cpool_self(),_Cthread_self(),index);
-			}
-#endif
-			Cthread_mutex_unlock_ext(current->state_cthread_structure[index]);
 			  
 		}
 #ifndef _WIN32
@@ -1995,135 +2040,124 @@ int DLL_DECL Cpool_assign_ext(poolnb,pooladdr,startroutine,arg,timeout)
 					}
 				}
 #ifdef CPOOL_DEBUG
-				if (current->forceid == -1) {
-					if (Cpool_debug != 0) {
-						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : try-lock on current->state_cthread_structure[%d]\n",
-							_Cpool_self(),_Cthread_self(), i);
-					}
-				} else {
-					if (Cpool_debug != 0) {
-						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : lock on current->state_cthread_structure[%d], timeout=%d\n",
-							_Cpool_self(),_Cthread_self(), i, timeout);
-					}
+				if (Cpool_debug != 0) {
+					log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : lock on current->state_cthread_structure[%d]\n",
+						_Cpool_self(),_Cthread_self(), i);
 				}
 #endif
-				if (((current->forceid == -1) ? Cthread_mutex_trylock_ext(current->state_cthread_structure[i]) : Cthread_mutex_timedlock_ext(current->state_cthread_structure[i], timeout)) == 0) {
+				if (Cthread_mutex_lock_ext(current->state_cthread_structure[i]) != 0) {
 #ifdef CPOOL_DEBUG
 					if (Cpool_debug != 0) {
-						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : current->state[%d]=%d, current->forceid=%d\n",
-							_Cpool_self(),_Cthread_self(), i, current->state[i], current->forceid);
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : lock on current->state_cthread_structure[%d] error: %s\n",
+							_Cpool_self(),_Cthread_self(), i, strerror(errno));
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : un-lock on lock_parent_cthread_structure\n",
+							_Cpool_self(),_Cthread_self());
 					}
 #endif
-					if (current->state[i] == 0) {
+					Cthread_mutex_unlock_ext(lock_parent_cthread_structure);
+					return(-1);
+				}
+#ifdef CPOOL_DEBUG
+				if (Cpool_debug != 0) {
+					log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : current->state[%d]=%d, current->forceid=%d\n",
+						_Cpool_self(),_Cthread_self(), i, current->state[i], current->forceid);
+				}
+#endif
+				if (current->state[i] == 0) {
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : Found thread at index %d\n",
+							_Cpool_self(),_Cthread_self(),i);
+					}
+#endif
+						
+					/* Okay: We change the predicate */
+					/* value on which the thread is  */
+					/* waiting for to be changed     */
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : Setting current->state[%d] to 1\n",
+							_Cpool_self(),_Cthread_self(),i);
+					}
+#endif
+					current->state[i] = 1;
+					/* We put the routine and its    */
+					/* arguments                     */
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : Setting current->start = 0x%lx, i = %d, stored at 0x%lx\n",
+							_Cpool_self(),_Cthread_self(),
+							(unsigned long) current->start,i,
+							(unsigned long) (current->start + (i * sizeof(void *))));
+					}
+#endif
+					current->start[i] = (void *(*) _PROTO((void *))) startroutine;
+					current->arg[i] = (void *) arg;
+					/* We signal the thread          */
+						
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : Set current->forceid to -1\n",
+							_Cpool_self(),_Cthread_self());
+					}
+#endif
+					/* ... We reset any call to Cpool_next_index */
+					current->forceid = -1;
+						
+					/* Note: Only one thread can wait on that condition variable */
+					/* It means that we are sure that Cthread will internal use */
+					/* a pure cond_signal instead of a cond_broadcast - this is */
+					/* better */
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : cond-signal on current->state_cthread_structure[%d]\n",
+							_Cpool_self(),_Cthread_self(), i);
+					}
+#endif
+					if (Cthread_cond_signal_ext(current->state_cthread_structure[i]) != 0) {
 #ifdef CPOOL_DEBUG
 						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : Found thread at index %d\n",
+							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : Oooups... Cthread_cond_signal_ext(current->state_cthread_structure[%d]) error (%s)\n",
+								_Cpool_self(),_Cthread_self(),i,strerror(errno));
+						}
+#endif
+#ifdef CPOOL_DEBUG
+						if (Cpool_debug != 0) {
+							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : un-lock on current->state_cthread_structure[%d]\n",
 								_Cpool_self(),_Cthread_self(),i);
 						}
 #endif
-						
-						/* Okay: We change the predicate */
-						/* value on which the thread is  */
-						/* waiting for to be changed     */
+						Cthread_mutex_unlock_ext(current->state_cthread_structure[i]);
 #ifdef CPOOL_DEBUG
 						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : Setting current->state[%d] to 1\n",
-								_Cpool_self(),_Cthread_self(),i);
-						}
-#endif
-						current->state[i] = 1;
-						/* We put the routine and its    */
-						/* arguments                     */
-#ifdef CPOOL_DEBUG
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : Setting current->start = 0x%lx, i = %d, stored at 0x%lx\n",
-								_Cpool_self(),_Cthread_self(),
-								(unsigned long) current->start,i,
-								(unsigned long) (current->start + (i * sizeof(void *))));
-						}
-#endif
-						current->start[i] = (void *(*) _PROTO((void *))) startroutine;
-						current->arg[i] = (void *) arg;
-						/* We signal the thread          */
-						
-#ifdef CPOOL_DEBUG
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : Set current->forceid to -1\n",
+							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : un-lock on lock_parent_cthread_structure\n",
 								_Cpool_self(),_Cthread_self());
 						}
 #endif
-						/* ... We reset any call to Cpool_next_index */
-						current->forceid = -1;
+						Cthread_mutex_unlock_ext(lock_parent_cthread_structure);
+						return(-1);
+					}
 						
-						/* Note: Only one thread can wait on that condition variable */
-						/* It means that we are sure that Cthread will internal use */
-						/* a pure cond_signal instead of a cond_broadcast - this is */
-						/* better */
-#ifdef CPOOL_DEBUG
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : cond-signal on current->state_cthread_structure[%d]\n",
-								_Cpool_self(),_Cthread_self(), i);
-						}
-#endif
-						if (Cthread_cond_signal_ext(current->state_cthread_structure[i]) != 0) {
-#ifdef CPOOL_DEBUG
-							if (Cpool_debug != 0) {
-								log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : Oooups... Cthread_cond_signal_ext(current->state_cthread_structure[%d]) error (%s)\n",
-									_Cpool_self(),_Cthread_self(),i,strerror(errno));
-							}
-#endif
-#ifdef CPOOL_DEBUG
-							if (Cpool_debug != 0) {
-								log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : un-lock on current->state_cthread_structure[%d]\n",
-									_Cpool_self(),_Cthread_self(),i);
-							}
-#endif
-							Cthread_mutex_unlock_ext(current->state_cthread_structure[i]);
-#ifdef CPOOL_DEBUG
-							if (Cpool_debug != 0) {
-								log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : un-lock on lock_parent_cthread_structure\n",
-									_Cpool_self(),_Cthread_self());
-							}
-#endif
-							Cthread_mutex_unlock_ext(lock_parent_cthread_structure);
-							return(-1);
-						}
-						
-						/* We remember for the immediate out of this loop... */
-						found = 1;
+					/* We remember for the immediate out of this loop... */
+					found = 1;
 				  
-						/* And we exit of the loop       */
+					/* And we exit of the loop       */
 #ifdef CPOOL_DEBUG
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : un-lock on current->state_cthread_structure[%d]\n",
-								_Cpool_self(),_Cthread_self(), i);
-						}
-#endif
-						Cthread_mutex_unlock_ext(current->state_cthread_structure[i]);
-						break;
-					} else {
-#ifdef CPOOL_DEBUG
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : un-lock on current->state_cthread_structure[%d]\n",
-								_Cpool_self(),_Cthread_self(), i);
-						}
-#endif
-						Cthread_mutex_unlock_ext(current->state_cthread_structure[i]);
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : un-lock on current->state_cthread_structure[%d]\n",
+							_Cpool_self(),_Cthread_self(), i);
 					}
-#ifdef CPOOL_DEBUG
+#endif
+					Cthread_mutex_unlock_ext(current->state_cthread_structure[i]);
+					break;
 				} else {
-					if (current->forceid == -1) {
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : try-lock on current->state_cthread_structure[%d] failed: %s\n",
-								_Cpool_self(),_Cthread_self(), i, strerror(errno));
-						}
-					} else {
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : lock on current->state_cthread_structure[%d] failed: %s\n",
-								_Cpool_self(),_Cthread_self(), i, strerror(errno));
-						}
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_assign : un-lock on current->state_cthread_structure[%d]\n",
+							_Cpool_self(),_Cthread_self(), i);
 					}
 #endif
+					Cthread_mutex_unlock_ext(current->state_cthread_structure[i]);
 				}
 			}
 		  
@@ -2695,65 +2729,68 @@ int DLL_DECL Cpool_next_index_timeout_ext(poolnb,pooladdr,timeout)
 			for (i = 0; i < current->nbelem; i++) {
 #ifdef CPOOL_DEBUG
 				if (Cpool_debug != 0) {
-					log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : try-lock on current->state_cthread_structure[%d]\n",
+					log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : lock on current->state_cthread_structure[%d]\n",
 						_Cpool_self(),_Cthread_self(), i);
 				}
 #endif
-				if (Cthread_mutex_trylock_ext(current->state_cthread_structure[i]) == 0) {
+				if (Cthread_mutex_lock_ext(current->state_cthread_structure[i]) != 0) {
 #ifdef CPOOL_DEBUG
 					if (Cpool_debug != 0) {
-						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : current->state[%d] is %d\n",
-							_Cpool_self(),_Cthread_self(), poolnb, current->state[i]);
-					}
-#endif
-					if (current->state[i] == 0) {
-#ifdef CPOOL_DEBUG
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : Found thread at index %d\n",
-								_Cpool_self(),_Cthread_self(),i);
-						}
-#endif
-						
-						/* Okay: We will return this index */
-#ifdef CPOOL_DEBUG
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : un-lock on current->state_cthread_structure[%d], set current->forceid to %d\n",
-								_Cpool_self(),_Cthread_self(), i, i);
-						}
-#endif
-						Cthread_mutex_unlock_ext(current->state_cthread_structure[i]);
-						current->forceid = i;
-#ifdef CPOOL_DEBUG
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : un-lock on lock_parent_cthread_structure\n",
-								_Cpool_self(),_Cthread_self());
-						}
-#endif
-						
-						Cthread_mutex_unlock_ext(lock_parent_cthread_structure);
-#ifdef CPOOL_DEBUG
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : Return %d\n",
-								_Cpool_self(),_Cthread_self(),i);
-						}
-#endif
-						return(i);
-					} else {
-#ifdef CPOOL_DEBUG
-						if (Cpool_debug != 0) {
-							log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : un-lock on current->state_cthread_structure[%d]\n",
-								_Cpool_self(),_Cthread_self(), i);
-						}
-#endif
-						Cthread_mutex_unlock_ext(current->state_cthread_structure[i]);
-					}
-#ifdef CPOOL_DEBUG
-				} else {
-					if (Cpool_debug != 0) {
-						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : try-lock on current->state_cthread_structure[%d] error: %s\n",
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : lock on current->state_cthread_structure[%d] error: %s\n",
 							_Cpool_self(),_Cthread_self(), i, strerror(errno));
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : un-lock on lock_parent_cthread_structure\n",
+							_Cpool_self(),_Cthread_self());
 					}
 #endif
+					Cthread_mutex_unlock_ext(lock_parent_cthread_structure);
+					return(-1);
+				}
+#ifdef CPOOL_DEBUG
+				if (Cpool_debug != 0) {
+					log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : current->state[%d] is %d\n",
+						_Cpool_self(),_Cthread_self(), poolnb, current->state[i]);
+				}
+#endif
+				if (current->state[i] == 0) {
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : Found thread at index %d\n",
+							_Cpool_self(),_Cthread_self(),i);
+					}
+#endif
+						
+					/* Okay: We will return this index */
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : un-lock on current->state_cthread_structure[%d], set current->forceid to %d\n",
+							_Cpool_self(),_Cthread_self(), i, i);
+					}
+#endif
+					Cthread_mutex_unlock_ext(current->state_cthread_structure[i]);
+					current->forceid = i;
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : un-lock on lock_parent_cthread_structure\n",
+							_Cpool_self(),_Cthread_self());
+					}
+#endif
+						
+					Cthread_mutex_unlock_ext(lock_parent_cthread_structure);
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : Return %d\n",
+							_Cpool_self(),_Cthread_self(),i);
+					}
+#endif
+					return(i);
+				} else {
+#ifdef CPOOL_DEBUG
+					if (Cpool_debug != 0) {
+						log(LOG_INFO,"[Cpool  [%2d][%2d]] In Cpool_next_index_timeout_ext : un-lock on current->state_cthread_structure[%d]\n",
+							_Cpool_self(),_Cthread_self(), i);
+					}
+#endif
+					Cthread_mutex_unlock_ext(current->state_cthread_structure[i]);
 				}
 			}
 		  
