@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_libpq_ifce.c,v $ $Revision: 8692 $ $Date: 2013-04-22 11:33:34 +0200 (Mon, 22 Apr 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_libpq_ifce.c,v $ $Revision: 12420 $ $Date: 2016-06-01 16:17:24 +0200 (Wed, 01 Jun 2016) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -2675,6 +2675,46 @@ struct dpm_space_reserv *dpm_spcmd;
 	return (0);
 }
 
+
+
+dpm_update_uspcincr_spcmd_entry (dbfd, rec_addr, dpm_spcmd, uspaceincr)
+struct dpm_dbfd *dbfd;
+dpm_dbrec_addr *rec_addr;
+struct dpm_space_reserv *dpm_spcmd;
+long long int uspaceincr;
+{
+	char func[23];
+	char g_space_str[21];
+	PGresult *res;
+	char sql_stmt[1024];
+	char t_space_str[21];
+	char u_space_str[21];
+	static char update_stmt[] =
+		"UPDATE dpm_space_reserv SET \
+		S_GID = %d, T_SPACE = %s, G_SPACE = %s, U_SPACE = U_SPACE + (%s), \
+		POOLNAME = '%s', ASSIGN_TIME = %d, EXPIRE_TIME = %d, GROUPS = '%s' \
+		WHERE ROWID = %s";
+
+	strcpy (func, "dpm_update_spcmd_entry");
+	sprintf (sql_stmt, update_stmt,
+	    dpm_spcmd->s_gid,
+	    u64tostr (dpm_spcmd->t_space, t_space_str, -1),
+	    u64tostr (dpm_spcmd->g_space, g_space_str, -1),
+	    i64tostr (uspaceincr, u_space_str, -1),
+	    dpm_spcmd->poolname,
+	    dpm_spcmd->assign_time, dpm_spcmd->expire_time,
+	    dpm_spcmd->groups, *rec_addr);
+
+	res = PQexec (dbfd->Pconn, sql_stmt);
+	if (PQresultStatus (res) != PGRES_COMMAND_OK) {
+		dpm_libpq_error (func, "UPDATE", res, dbfd);
+		return (-1);
+	}
+	PQclear (res);
+	return (0);
+}
+
+
 dpm_update_xferreq_entry (dbfd, rec_addr, dpm_req)
 struct dpm_dbfd *dbfd;
 dpm_dbrec_addr *rec_addr;
