@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_main.c,v $ $Revision: 10850 $ $Date: 2014-07-30 14:45:25 +0200 (Wed, 30 Jul 2014) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_main.c,v $ $Revision: 12228 $ $Date: 2016-03-23 14:43:24 +0100 (Wed, 23 Mar 2016) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <signal.h>
@@ -39,6 +39,10 @@ static char sccsid[] = "@(#)$RCSfile: dpm_main.c,v $ $Revision: 10850 $ $Date: 2
 #include "serrno.h"
 #include "u64subr.h"
 
+
+#include "LcgdmDbPool_C.h"
+
+
 /*
  * Maximum number of sockets on which the server can listen
  */
@@ -55,6 +59,7 @@ char localhost[CA_MAXHOSTNAMELEN+1];
 int maxfds;
 int nbfthreads = DPM_NBFTHREADS;
 int nbsthreads = DPM_NBSTHREADS;
+int nbfthreadsmult = DPM_NBFTHREADSMULT;
 int reqctr;
 void *reqctr_lock;
 int nb_supported_protocols;
@@ -64,6 +69,7 @@ extern int Cdomainname (char *, int);
 extern int dpm_rm_onereplica (struct Cns_fileid *, char *, u_signed64, int *, struct dpm_dbfd *dbfd, char *, int, u_signed64, char *);
 extern int isTrustedHost (int, char *, char *, char *, char *);
 static int na_key = -1;
+struct dpm_srv_thread_info *dpm_srv_dbfd_info;
 
 static void dpm_delete_old_reqs(struct dpm_req_cleanup *);
 static int dpm_delete_req_range_limited(struct dpm_dbfd *, time_t, time_t, int, u_signed64 *);
@@ -213,6 +219,8 @@ struct main_args *main_args;
 	int c;
 	struct dpm_dbfd dbfd;
 	void *doit(void *);
+        void *doit_dbpool(void *);
+        
 	char *dp;
 	char *dpmconfigfile = NULL;
 	struct sockaddr_storage from;
@@ -306,6 +314,29 @@ struct main_args *main_args;
 
 	dpmlogit (func, "Local host: %s\n", localhost);
 
+	
+	
+	if (getenv("NBFTHREADSMULT")) {
+		nbfthreadsmult = strtol(getenv("NBFTHREADSMULT"), NULL, 10);
+
+		if (nbfthreadsmult <= 1) { /* This disables the dbpool enhancements completely */
+			dpmlogit (func, "Dbpool enhancements are now off.\n");
+			nbfthreadsmult = 1;
+		}
+		else {
+			if ((nbfthreadsmult < 0) || (nbfthreads*nbfthreadsmult >= DPM_MAXNBTHREADS) ) {
+				dpmlogit (func, "Invalid thread multiplier: %s\n", nbfthreadsmult);
+				exit (USERR);
+			}
+		}
+	}
+
+	dpmlogit (func, "Eff. DB connections: %d, Multiplied thread pool: %d\n", nbfthreads, nbfthreads*nbfthreadsmult);
+      
+	
+	
+	
+	
 	if ((p = getenv ("DPNS_HOST")))
 		strcpy (dpnshost, p);
 	dpmlogit (func, "DPNS_HOST = %s\n", dpnshost);
@@ -402,20 +433,44 @@ struct main_args *main_args;
 
 	/* Create a pool of threads for fast operations */
 
-	if ((ipool = Cpool_create_ext (nbfthreads, NULL, &pooladdr)) < 0) {
+	if ((ipool = Cpool_create_ext (nbfthreads*nbfthreadsmult, NULL, &pooladdr)) < 0) {
 		dpmlogit (func, DP002, "Cpool_create", sstrerror (serrno));
 		return (SYERR);
 	}
 	if ((dpm_srv_thread_info =
-	    calloc (nbfthreads, sizeof(struct dpm_srv_thread_info))) == NULL) {
+	    calloc (nbfthreads*nbfthreadsmult, sizeof(struct dpm_srv_thread_info))) == NULL) {
 		dpmlogit (func, DP002, "calloc", strerror(errno));
 		return (SYERR);
 	}
-	for (i = 0; i < nbfthreads; i++) {
+	for (i = 0; i < nbfthreads*nbfthreadsmult; i++) {
 		(dpm_srv_thread_info+i)->s = -1;
 		(dpm_srv_thread_info+i)->dbfd.idx = i;
 	}
 
+	
+        if (nbfthreadsmult > 1) { /* This is useless but easier to read */
+                if ((dpm_srv_dbfd_info =
+                        calloc (nbfthreads, sizeof(struct dpm_srv_thread_info))) == NULL) {
+                        dpmlogit (func, DP002, "calloc", strerror(errno));
+                        return (SYERR);
+                }
+                for (i = 0; i < nbfthreads; i++) {
+                        (dpm_srv_dbfd_info+i)->dbfd.idx = i;
+                        if (dpm_opendb (&(dpm_srv_dbfd_info+i)->dbfd) < 0) {
+                                c = serrno;
+                                dpmlogit (func, DP002, "db open error: %d\n", c);
+                                return (CONFERR);
+                        }
+                        (dpm_srv_dbfd_info+i)->db_open_done = 1;
+                        (dpm_srv_dbfd_info+i)->last_db_use = time (0);
+                        LcgdmDbPool_AddDbfd((void *)(dpm_srv_dbfd_info+i));
+                }
+        }
+
+        dpmlogit (func, "DB looks sane. Pools created.\n");
+
+
+	
 	FD_ZERO (&readmask);
 	FD_ZERO (&readfd);
 #if ! defined(_WIN32)
@@ -519,23 +574,57 @@ struct main_args *main_args;
 
 	while (1) {
 		if (being_shutdown) {
-			int nb_active_threads = 0;
-			for (i = 0; i < nbfthreads; i++) {
-				if ((dpm_srv_thread_info+i)->s >= 0) {
-					nb_active_threads++;
-					continue;
-				}
-				if ((dpm_srv_thread_info+i)->db_open_done)
-					(void) dpm_closedb (&(dpm_srv_thread_info+i)->dbfd);
+			
+			if (nbfthreadsmult > 1) {
+				int nb_active_threads = 0;
+				for (i = 0; i < nbfthreads*nbfthreadsmult; i++) {
+					/* Fantastic... these items are not protected since ever */
+					if ((dpm_srv_thread_info+i)->s >= 0) {
+						nb_active_threads++;
+						continue;
+					}       
+					if ((dpm_srv_thread_info+i)->db_open_done)
+						(void) dpm_closedb (&(dpm_srv_thread_info+i)->dbfd);
+				}	       
+				
+				if (nb_active_threads == 0) {
+					/* Cleanup various things, we are going to exit */
+#ifdef CSEC			     
+					/* Cleanup the Csec instances */
+					for (i = 0; i < nbfthreads*nbfthreadsmult; i++) {
+						(void) Csec_clearContext (&(dpm_srv_thread_info+i)->sec_ctx);
+					}       
+#endif				  
+
+					/* Close all the db instances */
+					for (i = 0; i < nbfthreads; i++) {
+						(void) dpm_closedb (&(dpm_srv_dbfd_info+i)->dbfd);
+					}       
+					
+					return (0);
+				}       
+			}       
+			else {
+				/* The old code, buggy and taboo */
+				int nb_active_threads = 0;
+				for (i = 0; i < nbfthreads; i++) {
+					/* Fantastic... these items are not protected since ever */
+					if ((dpm_srv_thread_info+i)->s >= 0) {
+						nb_active_threads++;
+						continue;
+					}
+					if ((dpm_srv_thread_info+i)->db_open_done)
+						(void) dpm_closedb (&(dpm_srv_thread_info+i)->dbfd);
 #ifdef CSEC
-				(void) Csec_clearContext (&(dpm_srv_thread_info+i)->sec_ctx);
+					(void) Csec_clearContext (&(dpm_srv_thread_info+i)->sec_ctx);
 #endif
-			}
-			if (nb_active_threads == 0) {
-				(void) Cthread_join (ms_tid, NULL);
-				(void) Cthread_join (rex_tid, NULL);
-				(void) dpm_wait4allgcs ();
-				return (0);
+				}
+				if (nb_active_threads == 0) {
+					(void) Cthread_join (ms_tid, NULL);
+					(void) Cthread_join (rex_tid, NULL);
+					(void) dpm_wait4allgcs ();
+					return (0);
+				}
 			}
 		}
 		for (i=0; i < num_listen_socks; ++i) {
@@ -555,11 +644,23 @@ struct main_args *main_args;
 						return (SYERR);
 				}
 				(dpm_srv_thread_info+thread_index)->s = rqfd;
-				if (Cpool_assign_ext (ipool, pooladdr, &doit,
-				    dpm_srv_thread_info+thread_index, 1) < 0) {
-					(dpm_srv_thread_info+thread_index)->s = -1;
-					dpmlogit (func, DP002, "Cpool_assign", sstrerror (serrno));
-					return (SYERR);
+				
+				if (nbfthreadsmult > 1) {
+					/* Use the dbpool */
+					if (Cpool_assign (ipool, &doit_dbpool,
+					    dpm_srv_thread_info+thread_index, 1) < 0) {
+						(dpm_srv_thread_info+thread_index)->s = -1;
+						dpmlogit (func, DP002, "Cpool_assign", sstrerror(serrno));
+						return (SYERR);
+					}
+				}
+				else {
+					if (Cpool_assign_ext (ipool, pooladdr, &doit,
+					dpm_srv_thread_info+thread_index, 1) < 0) {
+						(dpm_srv_thread_info+thread_index)->s = -1;
+						dpmlogit (func, DP002, "Cpool_assign", sstrerror (serrno));
+						return (SYERR);
+					}
 				}
 			}
 		}
@@ -590,6 +691,216 @@ char **argv;
 #endif
 }
 
+
+
+
+
+
+
+
+
+
+void *
+doit_dbpool(arg)
+void *arg;
+{
+	int alloced_gids = 0;
+	int c;
+	const char *clienthost = NULL;
+	const char *clientip;
+	const char *clientname;
+	char **fqan = NULL;
+	int magic;
+	int nbfqans = 0;
+	char *req_data;
+	size_t req_datalen;
+	int req_type = 0;
+	char reqbuf[REQBUFSZ-3*LONGSIZE];
+	int set_authorization_id = 1;
+	struct dpm_srv_thread_info *thip = (struct dpm_srv_thread_info *) arg;
+	struct dpm_srv_thread_info *dbfdok = NULL;
+	struct dpm_srv_thread_info dbsave,dbsave2,thipsave;
+	char *user_ca = NULL;
+	char *voname = NULL;
+
+	(void) Cgetnetaddress (thip->s, NULL, 0, &na_key, &clientip, &clientname, 0, 0);
+	if (clientip == NULL)
+		clientip = "unknown";
+	if (clientname == NULL)
+		clientname = "unknown";
+
+#ifdef CSEC
+	Csec_server_reinitContext (&thip->sec_ctx, CSEC_SERVICE_TYPE_HOST, NULL);
+	if (Csec_server_establishContext (&thip->sec_ctx, thip->s) < 0) {
+		dpmlogit (func, "[%s] (%s): Could not establish an authenticated connection: %s !\n",
+			clientip, clientname, Csec_getErrorMessageSummary (LOGBUFSZ-140));
+		sendrep (thip->s, DPM_RC, ESEC_NO_CONTEXT);
+		thip->s = -1;
+		return NULL;
+	}
+	thip->Csec_gids = NULL;
+	Csec_server_getClientId (&thip->sec_ctx, &thip->Csec_mech, &thip->Csec_auth_id);
+	if (strcmp (thip->Csec_mech, "ID") == 0 ||
+	    Csec_isIdAService (thip->Csec_mech, thip->Csec_auth_id) >= 0) {
+		if (isTrustedHost (thip->s, localhost, localdomain, "DPM", "TRUST")) {
+			if (Csec_server_getAuthorizationId (&thip->sec_ctx,
+			    &thip->Csec_mech, &thip->Csec_auth_id) < 0) {
+				thip->Csec_uid = 0;
+				thip->Csec_gid = 0;
+				thip->Csec_nbgids = 1;
+				thip->Csec_gids = &thip->Csec_gid;
+				set_authorization_id = 0;
+#ifndef VIRTUAL_ID
+			} else if (Csec_mapToLocalUser (thip->Csec_mech, thip->Csec_auth_id,
+			    NULL, 0, &thip->Csec_uid, &thip->Csec_gid) < 0) {
+				dpmlogit (func, "[%s] (%s): Could not map (%s,\"%s\") to local user: %s !\n",
+				    clientip, clientname, thip->Csec_mech, thip->Csec_auth_id, sstrerror (serrno));
+				sendrep (thip->s, DPM_RC, serrno);
+				thip->s = -1;
+				return NULL;
+#else
+			} else {	/* mapping will be done later */
+				thip->Csec_uid = (uid_t) -1;
+				thip->Csec_gid = (gid_t) -1;
+				alloced_gids = 1;
+#endif
+			}
+		} else {
+			dpmlogit (func, "[%s] (%s): Host is not trusted, identity provided was (%s,\"%s\")\n",
+			    clientip, clientname, thip->Csec_mech, thip->Csec_auth_id);
+			sendrep (thip->s, DPM_RC, EACCES);
+			thip->s = -1;
+			return NULL;
+		}
+#ifndef VIRTUAL_ID
+	} else if (Csec_mapToLocalUser (thip->Csec_mech, thip->Csec_auth_id,
+	    NULL, 0, &thip->Csec_uid, &thip->Csec_gid) < 0) {
+		dpmlogit (func, "[%s] (%s): Could not map (%s,\"%s\") to local user: %s !\n",
+		    clientip, clientname, thip->Csec_mech, thip->Csec_auth_id, sstrerror (serrno));
+		sendrep (thip->s, DPM_RC, serrno);
+		thip->s = -1;
+		return NULL;
+#else
+	} else {	/* mapping will be done later */
+		thip->Csec_uid = (uid_t) -1;
+		thip->Csec_gid = (gid_t) -1;
+		alloced_gids = 1;
+#endif
+	}
+#ifdef VIRTUAL_ID
+	if (thip->Csec_uid == -1) {
+		user_ca = Csec_server_get_client_ca (&thip->sec_ctx);
+#ifdef USE_VOMS
+		voname = Csec_server_get_client_vo (&thip->sec_ctx);
+		fqan = Csec_server_get_client_fqans (&thip->sec_ctx, &nbfqans);
+#endif
+		/* must reset VOMS pointers in Cns client context */
+		Cns_client_setAuthorizationId (thip->Csec_uid, thip->Csec_gid,
+		    thip->Csec_mech, thip->Csec_auth_id);
+
+		thip->Csec_nbgids = nbfqans ? nbfqans : 1;
+		if ((thip->Csec_gids =
+		    malloc (thip->Csec_nbgids * sizeof(gid_t))) == NULL) {
+			dpmlogit (func,
+			    "[%s] (%s): Could not allocate memory for gids, identity was (%s,\"%s\")\n",
+			    clientip, clientname, thip->Csec_mech, thip->Csec_auth_id);
+			sendrep (thip->s, DPM_RC, ENOMEM);
+			thip->s = -1;
+			return NULL;
+		}
+		if (voname && fqan && nbfqans)
+			Cns_client_setVOMS_data (voname, fqan, 1);
+		if (Cns_getidmapc (thip->Csec_auth_id, user_ca, nbfqans,
+		    (const char **)fqan, &thip->Csec_uid, thip->Csec_gids)) {
+			if (serrno != EACCES)
+				dpmlogit (func,
+				    "[%s] (%s): Could not get virtual id for (%s,\"%s\"): %s !\n",
+				    clientip, clientname, thip->Csec_mech, thip->Csec_auth_id,
+				    sstrerror (serrno));
+			sendrep (thip->s, DPM_RC, serrno);
+			thip->s = -1;
+			return NULL;
+		}
+		if (voname && fqan)
+			Cns_client_setVOMS_data (voname, fqan, nbfqans);
+	}
+#endif
+	if (set_authorization_id) {
+		Cns_client_setAuthorizationId (thip->Csec_uid, thip->Csec_gid,
+		    thip->Csec_mech, thip->Csec_auth_id);
+		if (voname && fqan)
+			Cns_client_setVOMS_data (voname, fqan, nbfqans);
+	} else
+		Cns_client_resetAuthorizationId ();
+#endif
+	req_data = reqbuf;
+	
+	
+	
+	if ((c = getreq (thip->s, &magic, &req_type, &req_data, &req_datalen,
+	    &clienthost)) == 0) {
+		/* Get a thip containing a good db connection
+		   Only the dbfd field will be useful */
+		dbfdok = (struct Cns_srv_thread_info *)LcgdmDbPool_GetDbfd();
+		if (!dbfdok) {
+			sendrep (thip->s, MSG_ERR, "no more db instances: %d\n", c);
+			netclose (thip->s);
+			thip->s = -1;
+			return (NULL);
+		}
+		
+		/* Save the db conn pool and the per thread structures */
+		dbsave = *dbfdok;
+		thipsave = *thip;
+		
+		/* Reset dbfdok keeping only db elements from db pool entry */
+		*dbfdok = *thip;
+		dbfdok->dbfd = dbsave.dbfd;
+		dbfdok->db_open_done = dbsave.db_open_done;
+		dbfdok->last_db_use = dbsave.last_db_use;
+		
+		c = procreq (magic, req_type, req_data, req_datalen, clienthost, dbfdok);
+		if (dbfdok->dbfd.tr_started) {
+			(void) dpm_end_tr (&dbfdok->dbfd);
+			dpmlogit (func, "Ended an unexpected database transaction\n");
+		}
+		
+		
+		dbsave2 = *dbfdok;
+		*dbfdok = dbsave;
+		dbfdok->dbfd = dbsave2.dbfd;
+		dbfdok->db_open_done = dbsave2.db_open_done;
+		dbfdok->last_db_use = dbsave2.last_db_use;
+		LcgdmDbPool_AddDbfd((void *) dbfdok);
+
+		*thip = dbsave2;
+		thip->dbfd = thipsave.dbfd;
+		thip->db_open_done = thipsave.db_open_done;
+		thip->last_db_use = thipsave.last_db_use;
+		
+		
+		if (req_data != reqbuf)
+			free (req_data);
+	} else {
+		dpmlogit (func, "[%s] (%s): Failure getting the request: %s\n",
+		    clientip, clientname, sstrerror(c));
+		if (c > 0)
+			sendrep (thip->s, DPM_RC, c);
+		else
+			netclose (thip->s);
+	}
+#ifdef CSEC
+	if (alloced_gids)
+		free (thip->Csec_gids);
+#endif
+	thip->s = -1;
+	return (NULL);
+}
+
+
+
+
+
 void *
 doit(arg)
 	void *arg;
