@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_mysql_ifce.c,v $ $Revision: 10220 $ $Date: 2014-03-05 17:38:40 +0100 (Wed, 05 Mar 2014) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_mysql_ifce.c,v $ $Revision: 12420 $ $Date: 2016-06-01 16:17:24 +0200 (Wed, 01 Jun 2016) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -3035,6 +3035,52 @@ struct dpm_space_reserv *dpm_spcmd;
 	return (0);
 }
 
+
+dpm_update_uspcincr_spcmd_entry (dbfd, rec_addr, dpm_spcmd, uspaceincr)
+struct dpm_dbfd *dbfd;
+dpm_dbrec_addr *rec_addr;
+struct dpm_space_reserv *dpm_spcmd;
+long long int uspaceincr;
+{
+	char escaped_groups[255*2+1];
+	char escaped_poolname[CA_MAXPOOLNAMELEN*2+1];
+	char func[23];
+	char g_space_str[21];
+	char sql_stmt[1024];
+	char t_space_str[21];
+	char u_space_str[21];
+	static char update_stmt[] =
+		"UPDATE dpm_space_reserv SET \
+		S_GID = %d, T_SPACE = %s, G_SPACE = %s, U_SPACE = U_SPACE + (%s), \
+		POOLNAME = '%s', ASSIGN_TIME = %d, EXPIRE_TIME = %d, GROUPS = '%s' \
+		WHERE ROWID = %s";
+
+	strcpy (func, "dpm_update_spcmd_entry");
+	
+	dpm_spcmd->u_space += uspaceincr;
+	
+	dm_escape_string (&dbfd->mysql, escaped_groups, dpm_spcmd->groups,
+	    strlen (dpm_spcmd->groups), sizeof(escaped_groups));
+	dm_escape_string (&dbfd->mysql, escaped_poolname, dpm_spcmd->poolname,
+	    strlen (dpm_spcmd->poolname), sizeof(escaped_poolname));
+	dm_snprintf (sql_stmt, sizeof(sql_stmt), update_stmt,
+	    dpm_spcmd->s_gid,
+	    u64tostr (dpm_spcmd->t_space, t_space_str, -1),
+	    u64tostr (dpm_spcmd->g_space, g_space_str, -1),
+	    i64tostr (uspaceincr, u_space_str, -1),
+	    escaped_poolname,
+	    dpm_spcmd->assign_time, dpm_spcmd->expire_time,
+	    escaped_groups, *rec_addr);
+
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
+		dpm_mysql_error (func, "UPDATE", dbfd);
+		return (-1);
+	}
+	return (0);
+}
+
+
+
 dpm_update_xferreq_entry (dbfd, rec_addr, dpm_req)
 struct dpm_dbfd *dbfd;
 dpm_dbrec_addr *rec_addr;
