@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_proclreq.c,v $ $Revision: 9741 $ $Date: 2013-11-27 15:27:33 +0100 (Wed, 27 Nov 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_proclreq.c,v $ $Revision: 12420 $ $Date: 2016-06-01 16:17:24 +0200 (Wed, 01 Jun 2016) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -1193,12 +1193,25 @@ gid_t *gids;
 				strcpy (errstring, pfr_entry.errstring);
 				goto upd_pfr;
 			}
+			
 			dpm_spcmd.u_space -= pfr_entry.requested_size;
-			if (dpm_update_spcmd_entry (&thip->dbfd, &rec_addrs, &dpm_spcmd) < 0) {
+			
+			/*
+			
+			if (dpm_update_uspcincr_spcmd_entry (&thip->dbfd, &rec_addrs, &dpm_spcmd) < 0) {
 				pfr_entry.status = DPM_FAILED | serrno;
 				dpm_abort_tr (&thip->dbfd);
 				goto upd_pfr;
 			}
+			*/
+			
+			
+			if (dpm_update_uspcincr_spcmd_entry (&thip->dbfd, &rec_addrs, &dpm_spcmd, -pfr_entry.requested_size) < 0) {
+				pfr_entry.status = DPM_FAILED | serrno;
+				dpm_abort_tr (&thip->dbfd);
+				goto upd_pfr;
+			}
+			
 			(void) dpm_end_tr (&thip->dbfd);
 			dpmlogit (func, "Updated space token %s: incr=%s, u_space=%s\n",
 			    pfr_entry.s_token, i64tostr (-pfr_entry.requested_size, tmpbuf0, 0),
