@@ -1,5 +1,5 @@
 /*
- * $Id: dpm_server.h 11263 2015-01-14 13:29:21Z dhsmith $
+ * $Id: dpm_server.h 12420 2016-06-01 14:17:24Z furano $
  */
 
 /*
@@ -8,7 +8,7 @@
  */
 
 /*
- * @(#)$RCSfile: dpm_server.h,v $ $Revision: 11263 $ $Date: 2015-01-14 14:29:21 +0100 (Wed, 14 Jan 2015) $ CERN IT-GD/CT Jean-Philippe Baud
+ * @(#)$RCSfile: dpm_server.h,v $ $Revision: 12420 $ $Date: 2016-06-01 16:17:24 +0200 (Wed, 01 Jun 2016) $ CERN IT-GD/CT Jean-Philippe Baud
  */
 
 #ifndef _DPM_SERVER_H
@@ -34,6 +34,7 @@
 #define DPM_MAXNBTHREADS 1000	/* maximum number of threads */
 #define DPM_NBFTHREADS	20	/* number of threads to process fast requests */
 #define DPM_NBSTHREADS	20	/* number of threads to process slow requests */
+#define DPM_NBFTHREADSMULT 5    /* Thread multiplier when pooling the db connections of the dpm daemon */
 #define DPM_MAXNBPOOLS	(DPM_MAXNBTHREADS-DPM_NBFTHREADS-DPM_NBSTHREADS-2)
 #define RETURN(x) \
 	{ \
@@ -306,6 +307,7 @@ EXTERN_C int dpm_update_pending_entry _PROTO((struct dpm_dbfd *, dpm_dbrec_addr
 EXTERN_C int dpm_update_pfr_entry _PROTO((struct dpm_dbfd *, dpm_dbrec_addr *, struct dpm_put_filereq *));
 EXTERN_C int dpm_update_pool_entry _PROTO((struct dpm_dbfd *, dpm_dbrec_addr *, struct dpm_pool *));
 EXTERN_C int dpm_update_spcmd_entry _PROTO((struct dpm_dbfd *, dpm_dbrec_addr *, struct dpm_space_reserv *));
+EXTERN_C int dpm_update_uspcincr_spcmd_entry _PROTO((struct dpm_dbfd *, dpm_dbrec_addr *, struct dpm_space_reserv *, signed64 uspaceincr));
 EXTERN_C int dpm_update_xferreq_entry _PROTO((struct dpm_dbfd *, dpm_dbrec_addr *, struct dpm_req *));
 EXTERN_C int dpm_updfreespace _PROTO((char *, signed64, char *, int));
 EXTERN_C int dpm_updpoolfreespace _PROTO((char *, signed64));
