@@ -4,7 +4,7 @@
  */
  
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: Cns_mysql_ifce.c,v $ $Revision: 8852 $ $Date: 2013-05-28 11:33:35 +0200 (Tue, 28 May 2013) $ CERN IT-DS/HSM Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: Cns_mysql_ifce.c,v $ $Revision: 10220 $ $Date: 2014-03-05 17:38:40 +0100 (Wed, 05 Mar 2014) $ CERN IT-DS/HSM Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -149,6 +149,9 @@ Cns_abort_tr(dbfd)
 struct Cns_dbfd *dbfd;
 {
 	(void) mysql_query (&dbfd->mysql, "ROLLBACK");
+	if (!dbfd->tr_started) {
+		nslogit ("Cns_abort_tr", "Aborted transaction when none had been started\n");
+	}
 	dbfd->tr_mode = 0;
 	dbfd->tr_started = 0;
 	return (0);
@@ -418,6 +421,9 @@ Cns_end_tr(dbfd)
 struct Cns_dbfd *dbfd;
 {
 	(void) mysql_query (&dbfd->mysql, "COMMIT");
+	if (!dbfd->tr_started) {
+		nslogit ("Cns_end_tr", "Ended transaction when none had been started\n");
+	}
 	dbfd->tr_mode = 0;
 	dbfd->tr_started = 0;
 	return (0);
