@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: send2nsd.c,v $ $Revision: 8204 $ $Date: 2013-01-30 20:01:21 +0100 (Wed, 30 Jan 2013) $ CERN IT-PDP/DM Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: send2nsd.c,v $ $Revision: 10222 $ $Date: 2014-03-05 22:37:15 +0100 (Wed, 05 Mar 2014) $ CERN IT-PDP/DM Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -15,6 +15,7 @@ static char sccsid[] = "@(#)$RCSfile: send2nsd.c,v $ $Revision: 8204 $ $Date: 20
 #if defined(_WIN32)
 #include <winsock2.h>
 #else
+#include <unistd.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <sys/socket.h>
@@ -30,6 +31,35 @@ static char sccsid[] = "@(#)$RCSfile: send2nsd.c,v $ $Revision: 8204 $ $Date: 20
 #include "net.h"
 #include "serrno.h"
 
+static struct addrinfo *lookup_host(char *func, const char *node, const char *service) {
+	struct addrinfo *aitop;
+	int gaierrno;
+	struct addrinfo hints;
+
+	memset (&hints, 0, sizeof(struct addrinfo));
+	hints.ai_family = PF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
+#ifdef AI_ADDRCONFIG
+	hints.ai_flags |= AI_ADDRCONFIG;
+#endif
+
+	gaierrno = Cgetaddrinfo (node, service, &hints, &aitop);
+
+	if (gaierrno !=0 && serrno == 0)
+		serrno = SENOSHOST;
+
+	if (gaierrno == EAI_NONAME) {
+		Cns_errmsg (func, NS009, "Host unknown:", node);
+		return (NULL);
+	} else if (gaierrno != 0) {
+		Cns_errmsg (func, "Error during lookup of %s: %s\n", node,
+			Cgai_strerror (gaierrno));
+		return (NULL);
+	}
+
+	return (aitop);
+}
+
 /* send2nsd - send a request to the name server and wait for the reply */
 
 send2nsdx(socketp, host, reqp, reql, user_repbuf, user_repbuf_len, repbuf2, nbstruct)
@@ -45,7 +75,7 @@ int *nbstruct;
 	int abortconnect;
 	int actual_replen = 0;
 	struct addrinfo *ai;
-	struct addrinfo *aitop;
+	struct addrinfo *aitop = NULL;
 	int alloced = 0;
 	int c;
 	char Cnshost[CA_MAXHOSTNAMELEN+1];
@@ -65,8 +95,6 @@ int *nbstruct;
 #if defined(NSTYPE_LFC)
 	struct Cns_filereplicas *lr;
 #endif
-	int gaierrno;
-	struct addrinfo hints;
 	int isconnected;
 	int *ls;
 	struct Cns_userinfo *lui;
@@ -132,26 +160,6 @@ int *nbstruct;
 			}
 		}
 
-		memset (&hints, 0, sizeof(struct addrinfo));
-		hints.ai_family = PF_UNSPEC;
-		hints.ai_socktype = SOCK_STREAM;
-#ifdef AI_ADDRCONFIG
-		hints.ai_flags |= AI_ADDRCONFIG;
-#endif
-
-		gaierrno = Cgetaddrinfo (Cnshost, strport, &hints, &aitop);
-
-		if (gaierrno !=0 && serrno == 0)
-			serrno = SENOSHOST;
-
-		if (gaierrno == EAI_NONAME) {
-			Cns_errmsg (func, NS009, "Host unknown:", Cnshost);
-			return (-1);
-		} else if (gaierrno != 0) {
-			Cns_errmsg (func, "Error during lookup of %s: %s\n", Cnshost, Cgai_strerror (gaierrno));
-			return (-1);
-		}
-
 		/* get retry environment variables */
 		if ((p = Cns_getenv (CNS_CONNTIMEOUT_ENV)) == NULL) {
 			timeout = DEFAULT_CONNTIMEOUT;
@@ -176,14 +184,18 @@ int *nbstruct;
 		save_serrno = 0;
 		/* retry as much as the user wishes */
 	        while (retrycnt <= nbretry && !isconnected && !abortconnect) {
-
 			if (retrycnt>0)
 				sleep (retryint);
 			++retrycnt;
 
-			if (neterrstr) {
-				free (neterrstr);
-				neterrstr = NULL;
+			if (aitop) { freeaddrinfo (aitop); aitop = NULL; }
+
+			if (neterrstr) { free (neterrstr); neterrstr = NULL; }
+
+			if (!(aitop = lookup_host (func, Cnshost, strport))) {
+				abortconnect = 1;
+				save_serrno = serrno;
+				continue;
 			}
 
 			for (ai = aitop; ai && !isconnected; ai = ai->ai_next) {
@@ -267,7 +279,8 @@ int *nbstruct;
 #endif
 		}
 
-		freeaddrinfo (aitop);
+		if (aitop) { freeaddrinfo (aitop); aitop = NULL; }
+
 		if (!isconnected) {
 			serrno = (save_serrno) ? save_serrno : SECOMERR;
 			if (!abortconnect) {
