@@ -1,5 +1,5 @@
 /*
- * $Id: rfio_serv.c 3765 2010-08-30 06:38:44Z baud $
+ * $Id: rfio_serv.c 9911 2014-01-13 14:23:40Z dhsmith $
  */
 
 /*
@@ -8,7 +8,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: rfio_serv.c,v $ $Revision: 3765 $ $Date: 2010-08-30 08:38:44 +0200 (Mon, 30 Aug 2010) $ CERN/IT/PDP/DM Frederic Hemmer";
+static char sccsid[] = "@(#)$RCSfile: rfio_serv.c,v $ $Revision: 9911 $ $Date: 2014-01-13 15:23:40 +0100 (Mon, 13 Jan 2014) $ CERN/IT/PDP/DM Frederic Hemmer";
 #endif /* not lint */
 
 /* rfio_serv.c  SHIFT remote file access super server                   */
@@ -957,19 +957,14 @@ char tmpbuf[21], tmpbuf2[21];
       if (mode) return(1); else  exit(1);
    }
    Csec_server_getClientId(&sec_ctx, &Csec_mech, &Csec_auth_id);
+   Csec_uid = Csec_gid = -1;
    if (strcmp (Csec_mech, "ID") == 0 ||
      Csec_isIdAService (Csec_mech, Csec_auth_id) >= 0) {
       if (isTrustedHost (s, localhost, localdomain, "RFIOD", "TRUST")) {
          if (Csec_server_getAuthorizationId (&sec_ctx, &Csec_mech, &Csec_auth_id) < 0) {
-            Csec_uid = 0;
-            Csec_gid = 0;
+            Csec_uid = stagersuperuser.pw_uid;
+            Csec_gid = stagersuperuser.pw_gid;
             set_authorization_id = 0;
-         } else if (Csec_mapToLocalUser (Csec_mech, Csec_auth_id,
-           NULL, 0, &Csec_uid, &Csec_gid) < 0) {
-            log (LOG_ERR, "Could not map (%s,\"%s\") to local user: %s !\n",
-              Csec_mech, Csec_auth_id, sstrerror (serrno));
-            closesocket(s);
-            if (mode) return(1); else  exit(1);
          }
       } else {
          log (LOG_ERR, "Host is not trusted, identity provided was (%s,\"%s\")\n",
@@ -977,14 +972,13 @@ char tmpbuf[21], tmpbuf2[21];
          closesocket(s);
          if (mode) return(1); else  exit(1);
       }
-   } else if (Csec_mapToLocalUser (Csec_mech, Csec_auth_id,
-     NULL, 0, &Csec_uid, &Csec_gid) < 0) {
-      log (LOG_ERR, "Could not map (%s,\"%s\") to local user: %s !\n",
-        Csec_mech, Csec_auth_id, sstrerror (serrno));
+   }
+   if (rfio_handle_store_virtualid (Csec_auth_id)<0) {
       closesocket(s);
       if (mode) return(1); else  exit(1);
    }
    log(LOG_INFO, "request by %s (%d,%d)\n", Csec_auth_id, Csec_uid, Csec_gid);
+
 #endif
 
    /*
