@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: srmv1_procreq.c,v $ $Revision: 8696 $ $Date: 2013-04-22 20:33:50 +0200 (Mon, 22 Apr 2013) $ CERN Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: srmv1_procreq.c,v $ $Revision: 9653 $ $Date: 2013-10-31 17:44:45 +0100 (Thu, 31 Oct 2013) $ CERN Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -902,15 +902,9 @@ int ns5__setFileStatus (struct soap *soap, int reqid, int fileid, char *state, s
 			repfilep->size = gfr_entry.actual_size;
 			repfilep->state = soap_strdup (soap, f_stat[gfr_entry.status >> 12]);
 			repfilep->fileId = i;
-			if (*gfr_entry.pfn) {
-				if (strcmp (gfr_entry.protocol, "rfio") == 0)
-					p = strchr (gfr_entry.pfn, ':') + 1;
-				else
-					p = gfr_entry.pfn;
-				sprintf (turl, "%s://%s/%s", gfr_entry.protocol,
-				    gfr_entry.server, p);
+			gfr_to_turl (&thip->dbfd, &gfr_entry, turl, sizeof(turl));
+			if (*turl) 
 				repfilep->TURL = soap_strdup (soap, turl);
-			}
 			i++;
 		}
 		(void) dpm_list_gfr_entry (&thip->dbfd, bol, r_token,
@@ -971,15 +965,9 @@ int ns5__setFileStatus (struct soap *soap, int reqid, int fileid, char *state, s
 			repfilep->size = (pfr_entry.actual_size == LLONG_MAX) ? 0 : pfr_entry.actual_size;
 			repfilep->state = soap_strdup (soap, f_stat[pfr_entry.status >> 12]);
 			repfilep->fileId = i;
-			if (*pfr_entry.pfn) {
-				if (strcmp (pfr_entry.protocol, "rfio") == 0)
-					p = strchr (pfr_entry.pfn, ':') + 1;
-				else
-					p = pfr_entry.pfn;
-				sprintf (turl, "%s://%s/%s", pfr_entry.protocol,
-				    pfr_entry.server, p);
+			pfr_to_turl (&thip->dbfd, &pfr_entry, turl, sizeof(turl));
+			if (*turl)
 				repfilep->TURL = soap_strdup (soap, turl);
-			}
 			i++;
 		}
 		(void) dpm_list_pfr_entry (&thip->dbfd, bol, r_token,
@@ -1122,15 +1110,9 @@ int ns5__getRequestStatus (struct soap *soap, int reqid, struct ns5__getRequestS
 			repfilep->size = gfr_entry.actual_size;
 			repfilep->state = soap_strdup (soap, f_stat[gfr_entry.status >> 12]);
 			repfilep->fileId = i;
-			if (*gfr_entry.pfn) {
-				if (strcmp (gfr_entry.protocol, "rfio") == 0)
-					p = strchr (gfr_entry.pfn, ':') + 1;
-				else
-					p = gfr_entry.pfn;
-				sprintf (turl, "%s://%s/%s", gfr_entry.protocol,
-				    gfr_entry.server, p);
+			gfr_to_turl (&thip->dbfd, &gfr_entry, turl, sizeof(turl));
+			if (*turl)
 				repfilep->TURL = soap_strdup (soap, turl);
-			}
 			i++;
 		}
 		(void) dpm_list_gfr_entry (&thip->dbfd, bol, r_token,
@@ -1149,15 +1131,9 @@ int ns5__getRequestStatus (struct soap *soap, int reqid, struct ns5__getRequestS
 			repfilep->size = (pfr_entry.actual_size == LLONG_MAX) ? 0 : pfr_entry.actual_size;
 			repfilep->state = soap_strdup (soap, f_stat[pfr_entry.status >> 12]);
 			repfilep->fileId = i;
-			if (*pfr_entry.pfn) {
-				if (strcmp (pfr_entry.protocol, "rfio") == 0)
-					p = strchr (pfr_entry.pfn, ':') + 1;
-				else
-					p = pfr_entry.pfn;
-				sprintf (turl, "%s://%s/%s", pfr_entry.protocol,
-				    pfr_entry.server, p);
+			pfr_to_turl (&thip->dbfd, &pfr_entry, turl, sizeof(turl));
+			if (*turl)
 				repfilep->TURL = soap_strdup (soap, turl);
-			}
 			i++;
 		}
 		(void) dpm_list_pfr_entry (&thip->dbfd, bol, r_token,
