@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: srmv2.c,v $ $Revision: 4496 $ $Date: 2011-06-20 15:57:02 +0200 (Mon, 20 Jun 2011) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: srmv2.c,v $ $Revision: 10669 $ $Date: 2014-06-18 09:07:05 +0200 (Wed, 18 Jun 2014) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <signal.h>
@@ -20,6 +20,7 @@ static char sccsid[] = "@(#)$RCSfile: srmv2.c,v $ $Revision: 4496 $ $Date: 2011-
 #include "cgsi_plugin.h"
 #include "dpm.h"
 #include "dpm_server.h"
+#include "dpm_util.h"
 #include "patchlevel.h"
 #include "serrno.h"
 #include "srm_server.h"
@@ -126,7 +127,7 @@ struct main_args *main_args;
 	signal (SIGXFSZ, SIG_IGN);
 #endif
 
-	soap_init (&soap);
+	soap_init2 (&soap, SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
 	soap.recv_timeout = SRM_TIMEOUT;
 	flags = CGSI_OPT_DELEG_FLAG;
 #ifdef VIRTUAL_ID
@@ -224,6 +225,10 @@ process_request(void *soap)
 		}
 	}
 
+	if (thip->dbfd.tr_started) {
+		(void) dpm_end_tr (&thip->dbfd);
+		srmlogit (func, "Ended an unexpected database transaction\n");
+	}
 	thip->s = -1;
 	soap_end ((struct soap *)soap);
 	soap_done ((struct soap *)soap);
