@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: srmv2_xferreq.c,v $ $Revision: 8696 $ $Date: 2013-04-22 20:33:50 +0200 (Mon, 22 Apr 2013) $ CERN Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: srmv2_xferreq.c,v $ $Revision: 9653 $ $Date: 2013-10-31 17:44:45 +0100 (Thu, 31 Oct 2013) $ CERN Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <string.h>
@@ -80,7 +80,7 @@ marshall_CPR (struct soap *soap, struct ns1__TCopyRequestFileStatus *repfilep, s
 	return (0);
 }
 
-marshall_GFR (struct soap *soap, struct ns1__TGetRequestFileStatus *repfilep, struct dpm_get_filereq *gfr_entry)
+marshall_GFR (struct soap *soap, struct dpm_dbfd *dbfd, struct ns1__TGetRequestFileStatus *repfilep, struct dpm_get_filereq *gfr_entry)
 {
 	time_t current_time = time(0);
 	char *p;
@@ -98,19 +98,15 @@ marshall_GFR (struct soap *soap, struct ns1__TGetRequestFileStatus *repfilep, st
 	    dpmstatus2statuscode (gfr_entry->status, 1, gfr_entry->lifetime);
 	if (*gfr_entry->errstring)
 		repfilep->status->explanation = soap_strdup (soap, gfr_entry->errstring);
-	if (*gfr_entry->pfn) {
-		if (strcmp (gfr_entry->protocol, "rfio") == 0)
-			p = strchr (gfr_entry->pfn, ':') + 1;
-		else
-			p = gfr_entry->pfn;
-		sprintf (turl, "%s://%s/%s", gfr_entry->protocol, gfr_entry->server, p);
+	gfr_to_turl (dbfd, gfr_entry, turl, sizeof(turl));
+	if (*turl) {
 		if ((repfilep->transferURL = soap_malloc (soap, sizeof(struct ns1__TTURL))))
 			repfilep->transferURL->value = soap_strdup (soap, turl);
 	}
 	return (0);
 }
 
-marshall_PFR (struct soap *soap, struct ns1__TPutRequestFileStatus *repfilep, struct dpm_put_filereq *pfr_entry)
+marshall_PFR (struct soap *soap, struct dpm_dbfd *dbfd, struct ns1__TPutRequestFileStatus *repfilep, struct dpm_put_filereq *pfr_entry)
 {
 	time_t current_time = time(0);
 	char *p;
@@ -129,12 +125,8 @@ marshall_PFR (struct soap *soap, struct ns1__TPutRequestFileStatus *repfilep, st
 	    dpmstatus2statuscode (pfr_entry->status, 1, pfr_entry->lifetime);
 	if (*pfr_entry->errstring)
 		repfilep->status->explanation = soap_strdup (soap, pfr_entry->errstring);
-	if (*pfr_entry->pfn) {
-		if (strcmp (pfr_entry->protocol, "rfio") == 0)
-			p = strchr (pfr_entry->pfn, ':') + 1;
-		else
-			p = pfr_entry->pfn;
-		sprintf (turl, "%s://%s/%s", pfr_entry->protocol, pfr_entry->server, p);
+	pfr_to_turl (dbfd, pfr_entry, turl, sizeof(turl));
+	if (*turl) {
 		if ((repfilep->transferURL = soap_malloc (soap, sizeof(struct ns1__TTURL))))
 			repfilep->transferURL->value = soap_strdup (soap, turl);
 	}
@@ -1682,7 +1674,7 @@ ns1__srmStatusOfGetRequest (struct soap *soap, struct ns1__srmStatusOfGetRequest
 		while ((c = dpm_list_gfr_entry (&thip->dbfd, bol, req->requestToken->value,
 		    &gfr_entry, 0, NULL, 0, &dblistptr)) == 0) {
 			bol = 0;
-			if (marshall_GFR (soap, repp->arrayOfFileStatuses->getStatusArray[i],
+			if (marshall_GFR (soap, &thip->dbfd, repp->arrayOfFileStatuses->getStatusArray[i],
 			    &gfr_entry) < 0) {
 				(void) dpm_list_gfr_entry (&thip->dbfd, bol,
 				    req->requestToken->value, &gfr_entry,
@@ -1711,7 +1703,7 @@ ns1__srmStatusOfGetRequest (struct soap *soap, struct ns1__srmStatusOfGetRequest
 			}
 			sprintf (logbuf, "StatusOfGetRequest %s", gfr_entry.from_surl);
 			srm_logreq (func, logbuf);
-			if (marshall_GFR (soap, repp->arrayOfFileStatuses->getStatusArray[i],
+			if (marshall_GFR (soap, &thip->dbfd, repp->arrayOfFileStatuses->getStatusArray[i],
 			    &gfr_entry) < 0)
 				RETURN (SOAP_EOM);
 		}
@@ -1852,7 +1844,7 @@ ns1__srmStatusOfPutRequest (struct soap *soap, struct ns1__srmStatusOfPutRequest
 		while ((c = dpm_list_pfr_entry (&thip->dbfd, bol, req->requestToken->value,
 		    &pfr_entry, 0, NULL, 0, &dblistptr)) == 0) {
 			bol = 0;
-			if (marshall_PFR (soap, repp->arrayOfFileStatuses->putStatusArray[i],
+			if (marshall_PFR (soap, &thip->dbfd, repp->arrayOfFileStatuses->putStatusArray[i],
 			    &pfr_entry) < 0) {
 				(void) dpm_list_pfr_entry (&thip->dbfd, bol,
 				    req->requestToken->value, &pfr_entry,
@@ -1881,7 +1873,7 @@ ns1__srmStatusOfPutRequest (struct soap *soap, struct ns1__srmStatusOfPutRequest
 			}
 			sprintf (logbuf, "StatusOfPutRequest %s", pfr_entry.to_surl);
 			srm_logreq (func, logbuf);
-			if (marshall_PFR (soap, repp->arrayOfFileStatuses->putStatusArray[i],
+			if (marshall_PFR (soap, &thip->dbfd, repp->arrayOfFileStatuses->putStatusArray[i],
 			    &pfr_entry) < 0)
 				RETURN (SOAP_EOM);
 		}
